@@ -1,7 +1,25 @@
 // -*- C++ -*-
 
-#include "pomp_internal.h"
 #include <Rdefines.h>
+#include "pomp_internal.h"
+
+SEXP systematic_resampling(SEXP weights);
+void nosort_resamp(int nw, double *w, int np, int *p, int offset);
+
+SEXP systematic_resampling (SEXP weights)
+{
+  int n;
+  SEXP perm;
+
+  n = LENGTH(weights);
+  PROTECT(perm = NEW_INTEGER(n));
+  PROTECT(weights = AS_NUMERIC(weights));
+  GetRNGstate();
+  nosort_resamp(n,REAL(weights),n,INTEGER(perm),1);
+  PutRNGstate();
+  UNPROTECT(2);
+  return(perm);
+}
 
 void nosort_resamp (int nw, double *w, int np, int *p, int offset)
 {
@@ -18,7 +36,7 @@ void nosort_resamp (int nw, double *w, int np, int *p, int offset)
 
   for (i = 0, j = 0; j < np; j++) {
     u += du;
-    // In the following line, the second test is needed to correct 
+    // In the following line, the second test is needed to correct
     // the infamous Bug of St. Patrick, 2017-03-17.
     while ((u > w[i]) && (i < nw-1)) i++;
     p[j] = i;
@@ -27,17 +45,3 @@ void nosort_resamp (int nw, double *w, int np, int *p, int offset)
     for (j = 0; j < np; j++) p[j] += offset;
 
 }
-
-SEXP systematic_resampling (SEXP weights)
-{
-  int n;
-  SEXP perm;
-
-  n = LENGTH(weights);
-  PROTECT(perm = NEW_INTEGER(n));
-  GetRNGstate();
-  nosort_resamp(n,REAL(weights),n,INTEGER(perm),1);
-  PutRNGstate();
-  UNPROTECT(1);
-  return(perm);
-}
